# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.

import os
from pathlib import Path
from shutil import copystat, copy2
from typing import Any, Union
from jinja2 import Environment, FileSystemLoader
from stat import S_IWUSR as OWNER_WRITE_PERMISSION


def render_template(
    root: Union[str, os.PathLike], file: Union[str, os.PathLike], **kwargs: Any
) -> None:
    env = Environment(loader=FileSystemLoader(root))
    template = env.get_template(str(file))
    content = template.render(kwargs)

    path_obj = Path(root) / file
    render_path = path_obj.with_suffix("") if path_obj.suffix == ".tmpl" else path_obj

    if path_obj.suffix == ".tmpl":
        path_obj.rename(render_path)

    render_path.write_text(content, "utf8")


def copytree(src: Path, dst: Path, project_name: str):
    import knext

    template_dir = os.path.join(knext.__path__[0], "templates")
    src = Path(template_dir) / src
    names = [x.name for x in src.iterdir()]

    if not dst.exists():
        dst.mkdir(parents=True)

    for name in names:
        _name = name.replace("${project}", project_name)
        src_name = src / name
        dst_name = dst / _name
        if src_name.is_dir():
            copytree(src_name, dst_name, project_name)
        else:
            copy2(src_name, dst_name)
            _make_writable(dst_name)

    copystat(src, dst)
    _make_writable(dst)


def _make_writable(path):
    current_permissions = os.stat(path).st_mode
    os.chmod(path, current_permissions | OWNER_WRITE_PERMISSION)



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.

import json
from typing import Dict, List, Sequence

from knext.common.base.client import Client
from knext.common.base.runnable import Input, Output
from knext.builder.component.base import SPGExtractor
from knext.builder.operator.spg_record import SPGRecord
from knext.builder import rest
from knext.builder.operator.op import PromptOp, ExtractOp
from nn4k.invoker import NNInvoker


class LLMBasedExtractor(SPGExtractor):
    """A Builder Component that extracting structured data from long texts by invoking large language model.

    Examples:
        prompt_op = REPrompt(
                    spg_type_name=Medical.Disease,
                    property_names=[
                        Medical.Disease.complication,
                        Medical.Disease.commonSymptom,
                    ]
                )
        extract = LLMBasedExtractor(
                    llm=NNInvoker.from_config("./config.json"),
                    prompt_ops=[prompt_op]
                )

    """

    """Knowledge extract operator of this component."""
    llm: NNInvoker
    """PromptOps."""
    prompt_ops: List[PromptOp]

    max_retry_times: int = 3
    debug: bool = False

    @property
    def input_types(self) -> Input:
        return Dict[str, str]

    @property
    def output_types(self) -> Output:
        return SPGRecord

    def invoke(self, input: Input) -> Sequence[Output]:
        raise NotImplementedError(
            f"{self.__class__.__name__} does not support being invoked separately."
        )

    def submit(self):
        raise NotImplementedError(
            f"{self.__class__.__name__} does not support being submitted separately."
        )

    def to_rest(self):
        """Transforms `LLMBasedExtractor` to REST model `ExtractNodeConfig`."""
        params = dict()
        params["model_config"] = json.dumps(self.llm.init_args)
        params["prompt_config"] = json.dumps(
            [Client.serialize(op.to_rest()) for op in self.prompt_ops]
        )
        from knext.builder.operator.builtin.online_runner import _BuiltInOnlineExtractor

        extract_op = _BuiltInOnlineExtractor(params)
        extract_op.params["max_retry_times"] = str(self.max_retry_times)
        extract_op.params["debug"] = "DEBUG" if self.debug else None
        config = rest.UserDefinedExtractNodeConfig(operator_config=extract_op.to_rest())

        return rest.Node(**super().to_dict(), node_config=config)

    @classmethod
    def from_rest(cls, node: rest.Node):
        pass


class UserDefinedExtractor(SPGExtractor):
    """A Process Component that transforming unstructured data into structured data by extract operator.

    Examples:
        extract = UserDefinedExtractor(
                    extract_op=DemoExtractOp(params={"config": "1"})
                )

    """

    """Knowledge extract operator of this component."""
    extract_op: ExtractOp

    @property
    def input_types(self) -> Input:
        return Dict[str, str]

    @property
    def output_types(self) -> Output:
        return Dict[str, str]

    def invoke(self, input: Input) -> Sequence[Output]:
        raise NotImplementedError(
            f"{self.__class__.__name__} does not support being invoked separately."
        )

    def submit(self):
        raise NotImplementedError(
            f"{self.__class__.__name__} does not support being submitted separately."
        )

    def to_rest(self):
        """Transforms `UserDefinedExtractor` to REST model `UserDefinedExtractNodeConfig`."""
        operator_config = self.extract_op.to_rest()
        config = rest.UserDefinedExtractNodeConfig(operator_config=operator_config)

        return rest.Node(**super().to_dict(), node_config=config)

    @classmethod
    def from_rest(cls, node: rest.Node):
        return cls()



# -*- coding: utf-8 -*-
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.
from enum import Enum
from typing import Union, Dict, List, Tuple, Sequence, Optional, Any

from knext.builder import rest
from knext.schema.model.base import BaseSpgType
from knext.schema.client import SchemaClient
from knext.common.base.runnable import Input, Output

from knext.schema.model.schema_helper import (
    SPGTypeName,
    PropertyName,
    RelationName,
    TripletName,
    SubPropertyName,
)
from knext.builder.component.base import Mapping
from knext.builder.operator.op import LinkOp, FuseOp, PredictOp
from knext.builder.operator.spg_record import SPGRecord


class LinkingStrategyEnum(str, Enum):
    IDEquals = "ID_EQUALS"


class FusingStrategyEnum(str, Enum):
    Overwrite = "OVERWRITE"


class PredictingStrategyEnum(str, Enum):
    pass


class MappingTypeEnum(str, Enum):
    Property = "PROPERTY"
    Relation = "RELATION"
    SubProperty = "SUB_PROPERTY"
    SubRelation = "SUB_RELATION"


FusingStrategy = Union[FusingStrategyEnum, FuseOp]
LinkingStrategy = Union[LinkingStrategyEnum, LinkOp]
PredictingStrategy = Union[PredictingStrategyEnum, PredictOp]


class SPGTypeMapping(Mapping):
    """A Builder Component that mapping source field[UnresolvedRecord with column names and values]
    to target field[SPGRecord with entity/event/concept/standard type and properties].

    The UnresolvedRecord will go through the following execution processes and be converted into SPGRecord:
    1. Field Mapping
        Map the source field data to the schema attribute field of the target type.
    2. Object Linking
        Traverse all mapped properties.
        If the object type of property is not `BasicType`, execute the default `IDEqual` linking strategy:
            Query with the property value as the `id` of the object SPGType instance.
            If a corresponding SPGType instance exists, establish an SPO relationship between subject and object.

        If a LinkOp is bound to the object type, the chain pointing process of generating objects based on attribute values is executed.
        Based on the property values, link to
        Establish an SPO relationship between the current subject type and the attribute type based on the attribute values.

    3. Predicate Predicting
        For the linked properties
    4. Subject Fusing


    Args:
        spg_type_name: The SPG type name of subject import from SPGTypeHelper.
    Examples:
        mapping = SPGTypeMapping(
            spg_type_name=DEFAULT.App
        ).add_property_mapping("id", DEFAULT.App.id) \
            .add_property_mapping("name", DEFAULT.App.name) \
            .add_property_mapping("riskMark", DEFAULT.App.riskMark) \
            .add_predicting_field(DEFAULT.App.useCert)
    """

    """The target subject type name of this mapping component."""
    spg_type_name: SPGTypeName

    fusing_strategy: FusingStrategy = None

    _property_mapping: Dict[TripletName, Optional[str]] = dict()
    _relation_mapping: Dict[TripletName, Optional[str]] = dict()
    _sub_property_mapping: Dict[
        Tuple[TripletName, MappingTypeEnum], Dict[str, str]
    ] = dict()

    _filters: List[Tuple[str, str]] = list()

    _object_linking_strategies: Dict[TripletName, LinkingStrategy] = dict()

    _predicate_predicting_strategies: Dict[TripletName, PredictingStrategy] = dict()

    _current: Tuple[TripletName, MappingTypeEnum] = None

    schema_session: SchemaClient.SchemaSession = None

    spg_type: BaseSpgType = None

    def model_post_init(self, __context: Any) -> None:
        if not self.schema_session:
            self.schema_session = SchemaClient().create_session()
        if not self.spg_type:
            self.spg_type = self.schema_session.get(self.spg_type_name)

    @property
    def input_types(self) -> Input:
        return Union[Dict[str, str], SPGRecord]

    @property
    def output_types(self) -> Output:
        return SPGRecord

    @property
    def dependencies(self):
        dependencies = []
        for triplet_name in {**self._property_mapping, **self._relation_mapping}.keys():
            dependencies.append(triplet_name[2])
        return dependencies

    def add_property_mapping(
        self,
        source_name: str,
        target_name: PropertyName,
        target_type: SPGTypeName = None,
        linking_strategy: LinkingStrategy = None,
    ):
        if target_name not in self.spg_type.properties:
            raise ValueError(
                f"Property [{target_name}] does not exist in [{self.spg_type_name}]."
            )
        object_type_name = self.spg_type.properties[target_name].object_type_name
        if target_type:
            assert target_type == object_type_name, (
                f"The SPGType of Property [{target_name}] is [{object_type_name}], but [{target_type}] is given."
                f"Please check your schema config."
            )
        triplet_name = (self.spg_type_name, target_name, object_type_name)

        if linking_strategy:
            pass
        elif object_type_name in LinkOp.bind_schemas:
            op_name = LinkOp.bind_schemas[object_type_name]
            linking_strategy = LinkOp.by_name(op_name)()
        else:
            linking_strategy = None
        self._property_mapping[triplet_name] = source_name
        self._object_linking_strategies[triplet_name] = linking_strategy

        if not self._current:
            self._current = (triplet_name, MappingTypeEnum.SubProperty)
        return self

    def add_relation_mapping(
        self,
        source_name: str,
        target_name: PropertyName,
        target_type: SPGTypeName,
        linking_strategy: LinkingStrategy = None,
    ):
        relation_name = target_name + "_" + target_type
        if relation_name not in self.spg_type.relations:
            raise ValueError(
                f"Relation [{relation_name}] with ObjectType [target_type]"
                f" does not exist in [{self.spg_type_name}]."
            )
        triplet_name = (self.spg_type_name, target_name, target_type)
        if linking_strategy:
            pass
        elif target_type in LinkOp.bind_schemas:
            op_name = LinkOp.bind_schemas[target_type]
            linking_strategy = LinkOp.by_name(op_name)()
        else:
            linking_strategy = None
        self._relation_mapping[triplet_name] = source_name
        self._object_linking_strategies[triplet_name] = linking_strategy

        if not self._current:
            self._current = (triplet_name, MappingTypeEnum.SubRelation)
        return self

    def add_predicting_property(
        self,
        target_name: PropertyName,
        target_type: SPGTypeName = None,
        predicting_strategy: PredictingStrategy = None,
    ):
        if target_name not in self.spg_type.properties:
            raise ValueError(
                f"Property [{target_name}] does not exist in [{self.spg_type_name}]."
            )
        object_type_name = self.spg_type.properties[target_name].object_type_name
        if target_type:
            assert target_type == object_type_name, (
                f"The SPGType of Property [{target_name}] is [{object_type_name}], but [{target_type}] is given."
                f"Please check your schema config."
            )
        triplet_name = (self.spg_type_name, target_name, object_type_name)
        if predicting_strategy:
            pass
        elif triplet_name in PredictOp.bind_schemas:
            op_name = PredictOp.bind_schemas[triplet_name]
            predicting_strategy = PredictOp.by_name(op_name)()
        else:
            predicting_strategy = None
        self._property_mapping[triplet_name] = None
        self._predicate_predicting_strategies[triplet_name] = predicting_strategy

        return self

    def add_predicting_relation(
        self,
        target_name: RelationName,
        target_type: SPGTypeName,
        predicting_strategy: PredictingStrategy = None,
    ):
        relation_name = target_name + "_" + target_type
        if relation_name not in self.spg_type.relations:
            raise ValueError(
                f"Relation [{relation_name}] with ObjectType [target_type]"
                f" does not exist in [{self.spg_type_name}]."
            )
        triplet_name = (self.spg_type_name, target_name, target_type)

        if predicting_strategy:
            pass
        elif triplet_name in PredictOp.bind_schemas:
            op_name = PredictOp.bind_schemas[triplet_name]
            predicting_strategy = PredictOp.by_name(op_name)()
        else:
            predicting_strategy = None
        self._relation_mapping[triplet_name] = None
        self._predicate_predicting_strategies[triplet_name] = predicting_strategy
        return self

    def add_sub_property_mapping(self, source_name: str, target_name: SubPropertyName):
        if not self._current:
            raise ValueError(
                "Please add property or relation mapping before adding sub_property mapping."
            )
        sub_property_mapping = self._sub_property_mapping.get(self._current, {})
        sub_property_mapping.update({target_name: source_name})
        self._sub_property_mapping.update({self._current: sub_property_mapping})
        return self

    def add_filter(self, column_name: str, column_value: str):
        """Adds data filtering rule.
        Only the column that meets `column_name=column_value` will execute the mapping.

        :param column_name: The column name to be filtered.
        :param column_value: The column value to be filtered.
        :return: self
        """
        self._filters.append((column_name, column_value))
        return self

    def to_rest(self):
        """
        Transforms `SPGTypeMapping` to REST model `SpgTypeMappingNodeConfig`.
        """
        if not self._property_mapping and not self._relation_mapping:
            for _rel in self.spg_type.relations.values():
                if _rel.is_dynamic:
                    continue
                self.add_relation_mapping(_rel.name, _rel.name, _rel.object_type_name)
            for _prop in self.spg_type.properties.values():
                self.add_property_mapping(_prop.name, _prop.name)

        if "id" not in [
            triplet_name[1]
            for triplet_name in {
                **self._property_mapping,
                **self._relation_mapping,
            }.keys()
        ]:
            raise ValueError(
                f"Must include mapping to Property [id] in SPGTypeMapping({self.spg_type_name})."
            )

        mapping_filters = [
            rest.MappingFilter(column_name=name, column_value=value)
            for name, value in self._filters
        ]
        mapping_configs = []
        for triplet_name, src_name in self._property_mapping.items():
            if src_name:
                linking_strategy = self._object_linking_strategies.get(
                    triplet_name, None
                )
                if isinstance(linking_strategy, LinkOp):
                    strategy_config = rest.OperatorLinkingConfig(
                        operator_config=linking_strategy.to_rest()
                    )
                elif linking_strategy == LinkingStrategyEnum.IDEquals:
                    strategy_config = rest.IdEqualsLinkingConfig()
                elif not linking_strategy:
                    strategy_config = None
                else:
                    raise ValueError(f"Invalid linking_strategy [{linking_strategy}].")
            else:
                predicting_strategy = self._predicate_predicting_strategies.get(
                    triplet_name, None
                )
                if isinstance(predicting_strategy, PredictOp):
                    strategy_config = rest.OperatorPredictingConfig(
                        operator_config=predicting_strategy.to_rest()
                    )
                elif not predicting_strategy:
                    strategy_config = None
                else:
                    raise ValueError(
                        f"Invalid predicting_strategy [{predicting_strategy}]."
                    )
            mapping_configs.append(
                rest.MappingConfig(
                    source=src_name,
                    target=triplet_name[1],
                    strategy_config=strategy_config,
                    mapping_type=MappingTypeEnum.Property,
                )
            )

        for triplet_name, src_name in self._relation_mapping.items():
            if src_name:
                linking_strategy = self._object_linking_strategies.get(
                    triplet_name, None
                )
                if isinstance(linking_strategy, LinkOp):
                    strategy_config = rest.OperatorLinkingConfig(
                        operator_config=linking_strategy.to_rest()
                    )
                elif linking_strategy == LinkingStrategyEnum.IDEquals:
                    strategy_config = rest.IdEqualsLinkingConfig()
                elif not linking_strategy:
                    strategy_config = None
                else:
                    raise ValueError(f"Invalid linking_strategy [{linking_strategy}].")
            else:
                predicting_strategy = self._predicate_predicting_strategies.get(
                    triplet_name, None
                )
                if isinstance(predicting_strategy, PredictOp):
                    strategy_config = rest.OperatorPredictingConfig(
                        operator_config=predicting_strategy.to_rest()
                    )
                elif not predicting_strategy:
                    strategy_config = None
                else:
                    raise ValueError(
                        f"Invalid predicting_strategy [{predicting_strategy}]."
                    )

            mapping_configs.append(
                rest.MappingConfig(
                    source=src_name,
                    target=triplet_name[1] + "#" + triplet_name[2],
                    strategy_config=strategy_config,
                    mapping_type=MappingTypeEnum.Relation,
                )
            )

        if isinstance(self.fusing_strategy, FuseOp):
            fusing_config = rest.OperatorFusingConfig(
                operator_config=self.fusing_strategy.to_rest()
            )
        elif self.fusing_strategy == FusingStrategyEnum.Overwrite:
            fusing_config = rest.OverwriteFusingConfig()
        elif not self.fusing_strategy:
            if self.spg_type_name in FuseOp.bind_schemas:
                op_name = FuseOp.bind_schemas[self.spg_type_name]
                op = FuseOp.by_name(op_name)()
                fusing_config = rest.OperatorFusingConfig(operator_config=op.to_rest())
            else:
                fusing_config = None
        else:
            raise ValueError(
                f"Invalid fusing_strategy [{self.subject_fusing_strategy}]."
            )

        for (
            triplet_name,
            mapping_type,
        ), sub_mapping in self._sub_property_mapping.items():
            for tgt_name, src_name in sub_mapping.items():
                mapping_configs.append(
                    rest.MappingConfig(
                        source=src_name,
                        target=triplet_name[1] + "#" + triplet_name[2] + "#" + tgt_name,
                        strategy_config=None,
                        mapping_type=mapping_type,
                    )
                )

        config = rest.SpgTypeMappingNodeConfig(
            spg_type=self.spg_type_name,
            mapping_filters=mapping_filters,
            mapping_configs=mapping_configs,
            subject_fusing_config=fusing_config,
        )
        return rest.Node(**super().to_dict(), node_config=config)

    def invoke(self, input: Input) -> Sequence[Output]:
        raise NotImplementedError(
            f"`invoke` method is not currently supported for {self.__class__.__name__}."
        )

    @classmethod
    def from_rest(cls, rest_model):
        raise NotImplementedError(
            f"`from_rest` method is not currently supported for {cls.__name__}."
        )

    def submit(self):
        raise NotImplementedError(
            f"`submit` method is not currently supported for {self.__class__.__name__}."
        )


class RelationMapping(Mapping):
    """A Process Component that mapping data to relation type.

    Args:
        subject_name: The subject name import from SPGTypeHelper.
        predicate_name: The predicate name.
        object_name: The object name import from SPGTypeHelper.
    Examples:
        mapping = RelationMapping(
                    subject_name=DEFAULT.App,
                    predicate_name=DEFAULT.App.useCert,
                    object_name=DEFAULT.Cert,
                ).add_property_mapping("src_id", "srcId") \
                 .add_property_mapping("dst_id", "dstId")

    """

    """The SPG type names of (subject, predicate, object) triplet imported from SPGTypeHelper and PropertyHelper."""
    subject_name: SPGTypeName
    predicate_name: RelationName
    object_name: SPGTypeName

    _mapping: Dict[str, str] = dict()

    _filters: List[Tuple[str, str]] = list()

    def add_sub_property_mapping(self, source_name: str, target_name: str):
        """Adds a field mapping from source data to property of spg_type.

        :param source_name: The source field to be mapped.
        :param target_name: The target field to map the source field to.
        :return: self
        """
        self._mapping[target_name] = source_name
        return self

    def add_filter(self, column_name: str, column_value: str):
        """Adds data filtering rule.
        Only the column that meets `column_ame=column_value` will execute the mapping.

        :param column_name: The column name to be filtered.
        :param column_value: The column value to be filtered.
        :return: self
        """
        self._filters.append((column_name, column_value))
        return self

    def to_rest(self):
        """Transforms `RelationMappingComponent` to REST model `MappingNodeConfig`."""

        mapping_filters = [
            rest.MappingFilter(column_name=name, column_value=value)
            for name, value in self._filters
        ]
        mapping_configs = [
            rest.MappingConfig(source=src_name, target=tgt_name)
            for tgt_name, src_name in self._mapping.items()
        ]

        config = rest.RelationMappingNodeConfig(
            relation=f"{self.subject_name}_{self.predicate_name}_{self.object_name}",
            mapping_filters=mapping_filters,
            mapping_configs=mapping_configs,
        )
        return rest.Node(**super().to_dict(), node_config=config)

    @classmethod
    def from_rest(cls, node: rest.Node):
        pass

    def invoke(self, input: Input) -> Sequence[Output]:
        pass

    def submit(self):
        pass


class _SPGTypeMappings(Mapping):

    spg_type_mappings: List[SPGTypeMapping]

    def to_rest(self):
        config = Mapping.sort_by_dependency(self.spg_type_mappings)
        return rest.Node(**super().to_dict(), node_config=config)



# coding: utf-8
# Copyright 2023 OpenSPG Authors
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
# in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License
# is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied.


"""
    knext

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from knext.common.rest.configuration import Configuration


class BasicType(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        "basic_info": "StandardTypeBasicInfo",
        "parent_type_info": "ParentTypeInfo",
        "spg_type_enum": "str",
        "properties": "list[Property]",
        "relations": "list[Relation]",
        "advanced_config": "SpgTypeAdvancedConfig",
        "project_id": "int",
        "ontology_id": "OntologyId",
        "alter_operation": "str",
        "ext_info": "object",
        "basic_type": "str",
    }

    attribute_map = {
        "basic_info": "basicInfo",
        "parent_type_info": "parentTypeInfo",
        "spg_type_enum": "spgTypeEnum",
        "properties": "properties",
        "relations": "relations",
        "advanced_config": "advancedConfig",
        "project_id": "projectId",
        "ontology_id": "ontologyId",
        "alter_operation": "alterOperation",
        "ext_info": "extInfo",
        "basic_type": "basicType",
    }

    def __init__(
        self,
        basic_info=None,
        parent_type_info=None,
        spg_type_enum="BASIC_TYPE",
        properties=None,
        relations=None,
        advanced_config=None,
        project_id=None,
        ontology_id=None,
        alter_operation=None,
        ext_info=None,
        basic_type=None,
        local_vars_configuration=None,
    ):  # noqa: E501
        """BasicType - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._basic_info = None
        self._parent_type_info = None
        self._spg_type_enum = None
        self._properties = None
        self._relations = None
        self._advanced_config = None
        self._project_id = None
        self._ontology_id = None
        self._alter_operation = None
        self._ext_info = None
        self._basic_type = None
        self.discriminator = spg_type_enum

        if basic_info is not None:
            self.basic_info = basic_info
        if parent_type_info is not None:
            self.parent_type_info = parent_type_info
        self.spg_type_enum = spg_type_enum
        if properties is not None:
            self.properties = properties
        if relations is not None:
            self.relations = relations
        if advanced_config is not None:
            self.advanced_config = advanced_config
        if project_id is not None:
            self.project_id = project_id
        if ontology_id is not None:
            self.ontology_id = ontology_id
        if alter_operation is not None:
            self.alter_operation = alter_operation
        if ext_info is not None:
            self.ext_info = ext_info
        self.basic_type = basic_type

    @property
    def basic_info(self):
        """Gets the basic_info of this BasicType.  # noqa: E501


        :return: The basic_info of this BasicType.  # noqa: E501
        :rtype: StandardTypeBasicInfo
        """
        return self._basic_info

    @basic_info.setter
    def basic_info(self, basic_info):
        """Sets the basic_info of this BasicType.


        :param basic_info: The basic_info of this BasicType.  # noqa: E501
        :type: StandardTypeBasicInfo
        """

        self._basic_info = basic_info

    @property
    def parent_type_info(self):
        """Gets the parent_type_info of this BasicType.  # noqa: E501


        :return: The parent_type_info of this BasicType.  # noqa: E501
        :rtype: ParentTypeInfo
        """
        return self._parent_type_info

    @parent_type_info.setter
    def parent_type_info(self, parent_type_info):
        """Sets the parent_type_info of this BasicType.


        :param parent_type_info: The parent_type_info of this BasicType.  # noqa: E501
        :type: ParentTypeInfo
        """

        self._parent_type_info = parent_type_info

    @property
    def spg_type_enum(self):
        """Gets the spg_type_enum of this BasicType.  # noqa: E501


        :return: The spg_type_enum of this BasicType.  # noqa: E501
        :rtype: str
        """
        return self._spg_type_enum

    @spg_type_enum.setter
    def spg_type_enum(self, spg_type_enum):
        """Sets the spg_type_enum of this BasicType.


        :param spg_type_enum: The spg_type_enum of this BasicType.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation
            and spg_type_enum is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `spg_type_enum`, must not be `None`"
            )  # noqa: E501
        allowed_values = [
            "BASIC_TYPE",
            "ENTITY_TYPE",
            "CONCEPT_TYPE",
            "EVENT_TYPE",
            "STANDARD_TYPE",
        ]  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and spg_type_enum not in allowed_values
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `spg_type_enum` ({0}), must be one of {1}".format(  # noqa: E501
                    spg_type_enum, allowed_values
                )
            )

        self._spg_type_enum = spg_type_enum

    @property
    def properties(self):
        """Gets the properties of this BasicType.  # noqa: E501


        :return: The properties of this BasicType.  # noqa: E501
        :rtype: list[Property]
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """Sets the properties of this BasicType.


        :param properties: The properties of this BasicType.  # noqa: E501
        :type: list[Property]
        """

        self._properties = properties

    @property
    def relations(self):
        """Gets the relations of this BasicType.  # noqa: E501


        :return: The relations of this BasicType.  # noqa: E501
        :rtype: list[Relation]
        """
        return self._relations

    @relations.setter
    def relations(self, relations):
        """Sets the relations of this BasicType.


        :param relations: The relations of this BasicType.  # noqa: E501
        :type: list[Relation]
        """

        self._relations = relations

    @property
    def advanced_config(self):
        """Gets the advanced_config of this BasicType.  # noqa: E501


        :return: The advanced_config of this BasicType.  # noqa: E501
        :rtype: SpgTypeAdvancedConfig
        """
        return self._advanced_config

    @advanced_config.setter
    def advanced_config(self, advanced_config):
        """Sets the advanced_config of this BasicType.


        :param advanced_config: The advanced_config of this BasicType.  # noqa: E501
        :type: SpgTypeAdvancedConfig
        """

        self._advanced_config = advanced_config

    @property
    def project_id(self):
        """Gets the project_id of this BasicType.  # noqa: E501


        :return: The project_id of this BasicType.  # noqa: E501
        :rtype: int
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this BasicType.


        :param project_id: The project_id of this BasicType.  # noqa: E501
        :type: int
        """

        self._project_id = project_id

    @property
    def ontology_id(self):
        """Gets the ontology_id of this BasicType.  # noqa: E501


        :return: The ontology_id of this BasicType.  # noqa: E501
        :rtype: OntologyId
        """
        return self._ontology_id

    @ontology_id.setter
    def ontology_id(self, ontology_id):
        """Sets the ontology_id of this BasicType.


        :param ontology_id: The ontology_id of this BasicType.  # noqa: E501
        :type: OntologyId
        """

        self._ontology_id = ontology_id

    @property
    def alter_operation(self):
        """Gets the alter_operation of this BasicType.  # noqa: E501


        :return: The alter_operation of this BasicType.  # noqa: E501
        :rtype: str
        """
        return self._alter_operation

    @alter_operation.setter
    def alter_operation(self, alter_operation):
        """Sets the alter_operation of this BasicType.


        :param alter_operation: The alter_operation of this BasicType.  # noqa: E501
        :type: str
        """
        allowed_values = ["CREATE", "UPDATE", "DELETE"]  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and alter_operation not in allowed_values
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `alter_operation` ({0}), must be one of {1}".format(  # noqa: E501
                    alter_operation, allowed_values
                )
            )

        self._alter_operation = alter_operation

    @property
    def ext_info(self):
        """Gets the ext_info of this BasicType.  # noqa: E501


        :return: The ext_info of this BasicType.  # noqa: E501
        :rtype: object
        """
        return self._ext_info

    @ext_info.setter
    def ext_info(self, ext_info):
        """Sets the ext_info of this BasicType.


        :param ext_info: The ext_info of this BasicType.  # noqa: E501
        :type: object
        """

        self._ext_info = ext_info

    @property
    def basic_type(self):
        """Gets the basic_type of this BasicType.  # noqa: E501


        :return: The basic_type of this BasicType.  # noqa: E501
        :rtype: str
        """
        return self._basic_type

    @basic_type.setter
    def basic_type(self, basic_type):
        """Sets the basic_type of this BasicType.


        :param basic_type: The basic_type of this BasicType.  # noqa: E501
        :type: str
        """
        if (
            self.local_vars_configuration.client_side_validation and basic_type is None
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `basic_type`, must not be `None`"
            )  # noqa: E501
        allowed_values = ["TEXT", "LONG", "DOUBLE"]  # noqa: E501
        if (
            self.local_vars_configuration.client_side_validation
            and basic_type not in allowed_values
        ):  # noqa: E501
            raise ValueError(
                "Invalid value for `basic_type` ({0}), must be one of {1}".format(  # noqa: E501
                    basic_type, allowed_values
                )
            )

        self._basic_type = basic_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BasicType):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, BasicType):
            return True

        return self.to_dict() != other.to_dict()
